"use strict";
/*
 * Interface Definition
 * Keiya Chinen @ TwoGate inc.
 */
Object.defineProperty(exports, "__esModule", { value: true });
const ts_morph_1 = require("ts-morph");
const abstract_syntax_tree_1 = require("./abstract-syntax-tree");
const control_comment_1 = require("./control-comment");
const object_path_1 = require("./object-path");
const typedgate_error_1 = require("./typedgate-error");
class InterfaceDefinition {
    constructor(opts) {
        this.verbose = false;
        const project = new ts_morph_1.Project({
            tsConfigFilePath: opts.project.tsConfigFilePath
        });
        this.sourceFile = project.getSourceFileOrThrow(opts.sourceFilePath);
        this.targetData = opts.targetData;
    }
    compareToTarget() {
        const result = this.buildComparisonTree();
        return {
            valid: result,
            asts: this.asts,
        };
    }
    buildComparisonTree() {
        let asts = [];
        for (const [name, declarations] of this.sourceFile.getExportedDeclarations()) {
            const ast = declarations.reduce((r1, d) => {
                const entryDeclarations = d.getLeadingCommentRanges().reduce((r2, c) => {
                    try {
                        const comment = new control_comment_1.ControlComment(c.getText());
                        if (comment.isControlComment) {
                            switch (comment.getCommand()) {
                                case 'path':
                                    const objKeypath = comment.getArg();
                                    if (!objKeypath) {
                                        throw new typedgate_error_1.TypedgateError('Keypath not defined');
                                    }
                                    const op = new object_path_1.ObjectPath(objKeypath);
                                    const resolvedObj = op.traverse(this.targetData);
                                    if (!resolvedObj) {
                                        throw new typedgate_error_1.TypedgateError('Wrong Keypath');
                                    }
                                    else {
                                        if (d.getType().isInterface()) {
                                            r2.push(new abstract_syntax_tree_1.AbstractSyntaxTree(d, resolvedObj, op.path));
                                        }
                                        else if (d.getType().isClass()) {
                                            r2.push(new abstract_syntax_tree_1.AbstractSyntaxTree(d, resolvedObj, op.path));
                                        }
                                    }
                            }
                        }
                        return r2;
                    }
                    catch (e) {
                        if (e instanceof typedgate_error_1.TypedgateError) {
                            e.sourceFileName = c.getSourceFile().getFilePath();
                            e.sourceFilePos = c.getSourceFile().getLineAndColumnAtPos(c.getPos());
                            c.getSourceFile();
                            throw e;
                        }
                        else {
                            throw e;
                        }
                    }
                }, []);
                //console.log(entryDeclarations)
                if (entryDeclarations) {
                    r1 = r1.concat(entryDeclarations);
                }
                return r1;
            }, []);
            if (ast) {
                asts = asts.concat(ast);
            }
        }
        const results = asts.map((ast) => ast.validateDescendants(this.verbose)).every((result) => result);
        this.asts = asts;
        return results;
    }
}
exports.InterfaceDefinition = InterfaceDefinition;
//# sourceMappingURL=interface-definition.js.map